@@ -75,9 +75,17 @@ dwell_do_pointer_click (MTClosure *mt, gint x, gint y)
 	dwell_restore_single_click (mt);
 	break;
     case DWELL_CLICK_TYPE_DRAG:
-	SPI_generateMouseEvent (x, y, "b1p");
-	mt->dwell_drag_started = TRUE;
-	mt_cursor_set (GDK_FLEUR);
+	if (!mt->dwell_drag_started) {
+	    SPI_generateMouseEvent (x, y, "b1p");
+	    mt->dwell_drag_started = TRUE;
+	    mt_cursor_set (GDK_FLEUR);
+	}
+	else {
+	    SPI_generateMouseEvent (x, y, "b1r");
+	    mt->dwell_drag_started = FALSE;
+	    mt_cursor_set (GDK_LEFT_PTR);
+	    dwell_restore_single_click (mt);
+	}
 	break;
     case DWELL_CLICK_TYPE_RIGHT:
 	SPI_generateMouseEvent (x, y, "b3c");
@@ -208,45 +216,29 @@ dwell_stop_gesture (MTClosure *mt)
 static void
 dwell_timer_finished (MtTimer *timer, gpointer data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
     gint x, y;
 
     gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL);
     mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
 
-    /* stop active drag */
-    if (mt->dwell_drag_started) {
-	SPI_generateMouseEvent (x, y, "b1r");
-
-	mt->dwell_drag_started = FALSE;
-	mt_cursor_set (GDK_LEFT_PTR);
-	dwell_restore_single_click (mt);
-
-	return;
-    }
-
-    switch (mt->dwell_mode) {
-    case DWELL_MODE_CTW:
+    if (mt->dwell_mode == DWELL_MODE_CTW)
 	dwell_do_pointer_click (mt, x, y);
-	break;
-    case DWELL_MODE_GESTURE:
+    else if (mt->dwell_mode == DWELL_MODE_GESTURE) {
 	if (mt->dwell_gesture_started) {
 	    dwell_stop_gesture (mt);
-
 	    if (analyze_direction (mt, x, y))
 		dwell_do_pointer_click (mt, mt->pointer_x, mt->pointer_y);
 	}
 	else
 	    dwell_start_gesture (mt);
-    default:
-	break;
     }
 }
 
 static gboolean
 right_click_timeout (gpointer data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     SPI_generateMouseEvent (mt->pointer_x, mt->pointer_y, "b3c");
 
@@ -256,7 +248,7 @@ right_click_timeout (gpointer data)
 static void
 delay_timer_finished (MtTimer *timer, gpointer data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
 
@@ -270,7 +262,7 @@ delay_timer_finished (MtTimer *timer, gpointer data)
 static void
 spi_motion_event (const AccessibleEvent *event, void *data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     if (mt->dwell_enabled) {
 	if (!within_tolerance (mt, event->detail1, event->detail2) &&
@@ -279,7 +271,6 @@ spi_motion_event (const AccessibleEvent *event, void *data)
 	    mt->pointer_y = (gint) event->detail2;
 	    mt_timer_start (mt->dwell_timer);
 	}
-
 	if (mt->dwell_gesture_started) {
 	    if (mt->x_old > -1 && mt->y_old > -1)
 		draw_line (mt->pointer_x, mt->pointer_y, mt->x_old, mt->y_old);
@@ -291,7 +282,6 @@ spi_motion_event (const AccessibleEvent *event, void *data)
 	    mt->y_old = event->detail2;
 	}
     }
-
     if (mt_timer_is_running (mt->delay_timer)) {
 	if (!within_tolerance (mt, event->detail1, event->detail2)) {
 	    mt_timer_stop (mt->delay_timer);
@@ -303,15 +293,14 @@ spi_motion_event (const AccessibleEvent *event, void *data)
 static void
 spi_button_event (const AccessibleEvent *event, void *data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     if (g_str_equal (event->type, "mouse:button:1p")) {
 	if (mt->delay_enabled) {
-	    gdk_display_get_pointer (gdk_display_get_default (), NULL,
-				     &mt->pointer_x, &mt->pointer_y, NULL);
+	    mt->pointer_x = (gint) event->detail1;
+	    mt->pointer_y = (gint) event->detail2;
 	    mt_timer_start (mt->delay_timer);
 	}
-
 	if (mt->dwell_gesture_started)
 	    dwell_stop_gesture (mt);
     }
@@ -390,7 +379,7 @@ mt_update_cursor (MtCursor *cursor, MtTimer *timer, gdouble time)
 static void
 delay_timer_tick (MtTimer *timer, gdouble time, gpointer data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     if (mt->animate_cursor && mt->cursor != NULL)
 	mt_update_cursor (mt->cursor, timer, time);
@@ -399,7 +388,7 @@ delay_timer_tick (MtTimer *timer, gdouble time, gpointer data)
 static void
 dwell_timer_tick (MtTimer *timer, gdouble time, gpointer data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     if (mt->animate_cursor && mt->cursor != NULL)
 	mt_update_cursor (mt->cursor, timer, time);
@@ -409,7 +398,7 @@ static void
 cursor_cache_cleared (MtCursorManager *manager,
 		      gpointer         data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     mt->cursor = mt_cursor_manager_current_cursor (manager);
 }
@@ -419,7 +408,7 @@ cursor_changed (MtCursorManager *manager,
 		const gchar     *name,
 		gpointer         data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     if (!mt->dwell_gesture_started)
 	mt->override_cursor = !g_str_equal (name, "left_ptr");
@@ -439,7 +428,7 @@ gconf_value_changed (GConfClient *client,
 		     GConfValue  *value,
 		     gpointer     data)
 {
-    MTClosure *mt = (MTClosure *) data;
+    MTClosure *mt = data;
 
     if (g_str_equal (key, OPT_THRESHOLD) && value->type == GCONF_VALUE_INT)
 	mt->threshold = gconf_value_get_int (value);
@@ -541,6 +530,11 @@ accessibility_enabled (MTClosure *mt,
 		gnome_client_request_save (session, GNOME_SAVE_GLOBAL, TRUE,
 					   GNOME_INTERACT_ANY, FALSE, TRUE);
 	}
+	else {
+	    /* reset the selected option again */
+	    gconf_client_set_bool (mt->client, OPT_DELAY, FALSE, NULL);
+	    gconf_client_set_bool (mt->client, OPT_DWELL, FALSE, NULL);
+	}
 	return FALSE;
     }
     return TRUE;
@@ -596,25 +590,43 @@ mt_closure_free (MTClosure *mt)
 int
 main (int argc, char **argv)
 {
-    pid_t pid;
-    gboolean shutdown = FALSE, ctw = FALSE;
-    gchar *mode = NULL, *enable = NULL;
-    gint pos_x = -1, pos_y = -1;
+    pid_t    pid;
+    gboolean delay_click = FALSE;
+    gboolean dwell_click = FALSE;
+    gdouble  delay_time  = -1.;
+    gdouble  dwell_time  = -1.;
+    gboolean shutdown    = FALSE;
+    gboolean ctw         = FALSE;
+    gboolean animate     = FALSE;
+    gchar   *mode        = NULL;
+    gint     pos_x       = -1;
+    gint     pos_y       = -1;
+    gint     threshold   = -1;
     GOptionContext *context;
     GOptionEntry entries[] = {
-	{"enable", 'e', 0, G_OPTION_ARG_STRING, &enable,
-	    _("Enable mousetweaks feature"), "(dwell|delay)"},
+	{"enable-dwell", 0, 0, G_OPTION_ARG_NONE, &dwell_click,
+	    _("Enable dwell click"), 0},
+	{"enable-secondary", 0, 0, G_OPTION_ARG_NONE, &delay_click,
+	    _("Enable simulated secondary click"), 0},
+	{"dwell-time", 0, 0, G_OPTION_ARG_DOUBLE, &dwell_time,
+	    _("Time to wait before a dwell click"), "[0.5-3.0]"},
+	{"secondary-time", 0, 0, G_OPTION_ARG_DOUBLE, &delay_time,
+	    _("Time to wait before a simulated secondary click"), "[0.5-3.0]"},
 	{"dwell-mode", 'm', 0, G_OPTION_ARG_STRING, &mode,
-	    _("Use dwell mode"), "(window|gesture)"},
+	    _("Dwell mode to use"), "[window|gesture]"},
 	{"show-ctw", 'c', 0, G_OPTION_ARG_NONE, &ctw,
 	    _("Show click type window"), 0},
 	{"ctw-x", 'x', 0, G_OPTION_ARG_INT, &pos_x,
 	    _("Window x position"), 0},
 	{"ctw-y", 'y', 0, G_OPTION_ARG_INT, &pos_y,
 	    _("Window y position"), 0},
+	{"threshold", 't', 0, G_OPTION_ARG_INT, &threshold,
+	    _("Ignore small pointer movements"), "[0-30]"},
+	{"animate-cursor", 'a', 0, G_OPTION_ARG_NONE, &animate,
+	    _("Show elapsed time as cursor overlay"), 0},
 	{"shutdown", 's', 0, G_OPTION_ARG_NONE, &shutdown,
 	    _("Shut down mousetweaks"), 0},
-	{NULL}
+	{ NULL }
     };
 
     bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
@@ -636,12 +648,10 @@ main (int argc, char **argv)
 
 	return ret < 0 ? 1 : 0;
     }
-
     if ((pid = mt_pidfile_is_running ()) >= 0) {
 	g_print ("Daemon is already running. (PID %u)\n", pid);
 	return 1;
     }
-
     g_print ("Starting daemon.\n");
 
     if ((pid = fork ()) < 0) {
@@ -678,9 +688,6 @@ main (int argc, char **argv)
 
 	spi_status = SPI_init ();
 	if (!accessibility_enabled (mt, spi_status)) {
-	    /* disable options again */
-	    gconf_client_set_bool (mt->client, OPT_DELAY, FALSE, NULL);
-	    gconf_client_set_bool (mt->client, OPT_DWELL, FALSE, NULL);
 	    mt_closure_free (mt);
 	    goto FINISH;
 	}
@@ -693,21 +700,20 @@ main (int argc, char **argv)
 	SPI_registerGlobalEventListener (bl, "mouse:button:1r");
 
 	/* command-line options */
-	if (enable) {
-	    if (g_str_equal (enable, "dwell")) {
-		gconf_client_set_bool (mt->client, OPT_DELAY, FALSE, NULL);
-		gconf_client_set_bool (mt->client, OPT_DWELL, TRUE, NULL);
-	    }
-	    if (g_str_equal (enable, "delay")) {
-		gconf_client_set_bool (mt->client, OPT_DWELL, FALSE, NULL);
-		gconf_client_set_bool (mt->client, OPT_DELAY, TRUE, NULL);
-	    }
-	    g_free (enable);
-	}
-
+	if (dwell_click)
+	    gconf_client_set_bool (mt->client, OPT_DWELL, TRUE, NULL);
+	if (delay_click)
+	    gconf_client_set_bool (mt->client, OPT_DELAY, TRUE, NULL);
+	if (delay_time >= .5 && delay_time <= 3.)
+	    gconf_client_set_float (mt->client, OPT_DELAY_T, delay_time, NULL);
+	if (dwell_time >= .5 && dwell_time <= 3.)
+	    gconf_client_set_float (mt->client, OPT_DWELL_T, dwell_time, NULL);
+	if (threshold >= 0 && threshold <= 30)
+	    gconf_client_set_int (mt->client, OPT_THRESHOLD, threshold, NULL);
 	if (ctw)
 	    gconf_client_set_bool (mt->client, OPT_CTW, TRUE, NULL);
-
+	if (animate)
+	    gconf_client_set_bool (mt->client, OPT_ANIMATE, TRUE, NULL);
 	if (mode) {
 	    if (g_str_equal (mode, "gesture"))
 		gconf_client_set_int (mt->client, OPT_MODE,
@@ -738,7 +744,7 @@ main (int argc, char **argv)
 
 	CLEANUP:
 	    SPI_deregisterGlobalEventListenerAll (bl);
-	    AccessibleDeviceListener_unref (bl);
+	    AccessibleEventListener_unref (bl);
 	    SPI_deregisterGlobalEventListenerAll (ml);
 	    AccessibleEventListener_unref (ml);
 	    SPI_exit ();
